FOLDER = "G:/Shared drives/Research/Projects/Detector/PNR/Paderborn/Data/2024-01-15/"
FILENAME = "time_drift_data_125_2024_01_15_04_49_41.ttbin"

CALIBRATION_FILES = ['parameters_ch9.json', 'parameters_ch13.json']

LASER_CHANNEL = 5
FRONT_EDGE = [-9, -13]
REAR_EDGE = [9, 13]
